;_______________________________________________________________________*
;									*
;       Rotor-Interface mit PIC 16f84					*
;       Stand 20-feb-2002						*
;									*
;       LCD im nibble mode und wait for busyflag			*
;       RS232 mit Start durch Interrupt und FIFO Puffer			*
;       A/D-Wandler LTC1298						*
;									*
;                                                                       *
; Port Configurations :                                                 *
;                                                                       *
; RB0 : Serial Input  (9600 bps 8N1)	nur in                          *
; RB1 : RS-LCD und CLK A/D LTC 1298     nur out                         *
; RB2 : R/_W LCD                        nur out                         *
; RB3 : Enable LCD Data                 nur out                         *
; RB4 : DI/DO LTC 1298                  in/out                          *
; RB4 : LCD (B4) LSB                    in/out                          *
; RB5 : LCD (B5)                          "                             *
; RB6 : LCD (B6)                          "                             *
; RB7 : LCD (B7) MSB                      "                             *
;                                                                       *
; RA0 : Rotor UP                        nur out                         *
; RA1 : Rotor DOWN                      nur out                         *
; RA2 : Rotor LEFT                      nur out                         *
; RA3 : Rotor RIGHT                     nur out                         *
; RA4 : CS_ LTC 1298                    nur out                         *
;
;************************************************************************
;************************************************************************
; processor pic16f84                                                    *
; no code protection                                                    *
; power up timer enabled                                                *
; WDT disabled; gwr                                                     *
; XT oscillator  (4 MHz)                                                *
;************************************************************************
;***** Declaring and configuring a microcontroller *****

        PROCESSOR 16f84
        #include "p16f84.inc"
	
	__CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;***** Declaring variables *****

	Cblock 0x0C	; Beginning of RAM
; interrupt context save 
        w_
        status_
	fsr_
	pclath_
; LCD
	LCDbuf
	LCDtemp
; Print
        Pointer         ; Pointer on characters in message
; eeprom
        counter
; display 16bit binary value decimal on LCD and math16 routines
        TEMP0
        TEMP1
	TEMP2
	TEMP3
        LO
        HI
        LO_TEMP
	HI_TEMP
	HI_RESULT
	Bcheck
; A/D LTC1298
	ADC_Var
	ADLO_sav
	ADHI_sav
; RS232
        RS_TEMP1
        RS_TEMP2
        TXD
        RXD
        stringlen
        AZ_LO
        AZ_HI
        EL_LO
        EL_HI
        AZkorr_LO
        AZkorr_HI
        ELkorr_LO
        ELkorr_HI
        char_cnt
; Rotor
        AZ_char
        EL_char
        Rotor_Flag      ;U: b0=1,D: b1=1,L: b2=1,R: b3=1,AZ akt.b6=0,EL akt.b7=0
        Rotor_hyst      ;Hyterese fuer Bewegungen
; FIFO
        fifo_char
        fifo_len
        fifo_start
        fifo_base
        fifo_buffer:.16     ; wir brauchen 16 Zellen
        zzzz
        endc
;************************************************************************
;***** Declaring the hardware *****	

LCDtris equ     TRISB
LCDport equ     PORTB
ADtris	equ	TRISA
ADC_LO EQU LO
ADC_HI EQU HI
ramadr	equ        AZkorr_LO
#define TXport  PORTA,2
#define TXtris  TRISA,2
#define Data	PORTB,4
#define Clock	PORTB,1
#define CS	PORTA,4


;************************************************************************
;***** Structure of program memory *****

	ORG     0x00	; Reset vector
  	goto    Main

	ORG     0x04	; Interrupt vector
        goto    ISR    ; No interrupt routine

;************************************************************************

Messages		; Beginning of the messages

        movwf   PCL

    			; Display messages

Message1 dt "-- AOT-Rotor -- "
END_messages                    ; End of messages

;************************************************************************

shufflechar			; s/r schaufelt Reihenfolge der Digits um.
	movfw	char_cnt	; wir haben bis zu 3 Digits, das nieder- 
	addwf	PCL,f		; wertigste muss immer in TEMP2 stehen.
	goto	nullchars
	goto	onechar
	goto	twochars
	return
nullchars
	clrf	TEMP0
	clrf	TEMP1
	clrf	TEMP2
	return
onechar
	movfw	TEMP0
	movwf	TEMP2
	clrf	TEMP0
	clrf	TEMP1
	return
twochars
	movfw	TEMP1
	movwf	TEMP2
	movfw	TEMP0
	movwf	TEMP1
	clrf	TEMP0
	return

;************************************************************************

        #include "bank.inc"	; Assistant files
        #include "lcdbf.inc"
    	#include "print.inc"
        #include "rs232.inc"
	#include "ltc1298.inc"
	#include "lcdv16.inc"
        #include "lcdv08.inc"
        #include "asc2bin.inc"
        #include "math16.inc"
        #include "eeprom.inc"

;************************************************************************
;       Interrupt service fuer RS232

ISR     movwf   w_		; save complete context
        swapf   STATUS, w
        movwf   status_
        bcf     STATUS, RP0
	movfw	PCLATH
	movwf	pclath_
        movfw   FSR
	movwf	fsr_

        RECEIVE

ISRend
	movfw	fsr_		; restore context
        movwf   FSR
	movfw	pclath_
	movwf	PCLATH
        swapf   status_,w
        movwf   STATUS
        swapf   w_,f
        swapf   w_,w
        bcf     INTCON,INTF
        RETFIE

;************************************************************************

Main                    	; Beginning of the program

	bcf PORTB,2
        movlw   0xf0
	movwf	PORTA
	BANK1
	clrf	TRISA		; output on all pins
	movlw	0x01
	movwf	TRISB
	BANK0
        clrw                    ; ab EEPROM Adresse 0
        call    get_EElat       ; lies 4 Korrekturwerte fuer Rotor aus EEprom
	movlw	4		; und nun lies die Hysterese aus Adresse 4
	movwf	EEADR
	call	EE_Read
	movwf	Rotor_hyst
        clrf    AZ_char
        clrf    EL_char
        movlw   0xf0
        movwf   Rotor_Flag

	LCDinit		; LCD initialization
Main1
        RS232init       ; RS232 dtto
	
        PRINT   Messages, Message1, END_messages, Pointer, LCDw

;************************************************************************
Loop
        call    ADC_service
        call    Rotor_Move
        movfw   fifo_len
        btfsc   STATUS,Z
	goto	Loop

; rs232 service
        call    FIFOget
        movlw   'A'
        subwf   fifo_char,w
        bz      RS_azimut
        movlw   'E'
        subwf   fifo_char,w
        bz      RS_elevat
        movlw   'W'
        subwf   fifo_char,w
        bz      RS_aziele
        movlw   'a'
        subwf   fifo_char,w
        bz      RS_azkorr
        movlw   'e'
        subwf   fifo_char,w
        bz      RS_elkorr
        movlw   'Y'
        subwf   fifo_char,w
        bz      RS_hysterese
        movlw   'M'
        subwf   fifo_char,w
        bz      RS_message
        movlw   'K'
        subwf   fifo_char,w
        bz      show_korrektur
        goto    Loop

;************************************************************************

RS_azimut
        bcf     Rotor_Flag,6    ; AZ aktiv
        LCDcmd  0x88            ; 1.Zeile/8. Position
        call    SERV_input
        LCDchar ' '
        LCDchar ' '
        call    A_korr
        cpy16	LO,AZ_LO
        goto    Loop

; Korrektur fuer Antennenposition
A_korr
	cpy16	AZkorr_LO,LO_TEMP
        call    sub16
        movlw   .16
        movwf   LO_TEMP
        movlw   .14
        movwf   HI_TEMP
        btfsc   HI,7		; nur positive Werte erlaubt, daher gegebenenfalls
        call    add16		; Korrektur
        return

RS_elevat
        bcf     Rotor_Flag,7    ; EL aktiv
        LCDcmd  0x80            ; 1.Zeile/1. Position
        call    SERV_input
RS_elevat1
        LCDchar ' '
        LCDchar ' '
	cpy16	ELkorr_LO,LO_TEMP
        call    sub16
	cpy16	LO,EL_LO
        btfss   HI,7
        goto    Loop		; positives Ergebnis ist schon fertig
	call	add_180
	btfss	HI,7
	goto	ele_null_180	; positives Ergebnis fhrt zu Anschlag auf 0 oder 180 Grad
	call	add_180		; hier hatten wir einen negativen Offset, 
	cpy16	LO,EL_LO	; also von 180 bis 360, jetzt ist alles wieder gut
	goto	Loop

ele_null_180                    ; Endanschlag auf 0 oder 180 Grad
	movlw	.4
	movwf	LO_TEMP
	movlw	.3
	movwf	HI_TEMP
	call	sub16		; subtrahiere 90 Grad
	btfss	HI,7
	goto	ele_null	; erzwinge 0 Grad
	movlw	.8		; erzwinge 180 Grad
	movwf	EL_LO
	movlw	.7
	movwf	EL_HI	
	goto	Loop
ele_null
        clrf    EL_LO
        clrf    EL_HI
        goto    Loop

add_180
	movlw	.8
	movwf	LO_TEMP
	movlw	.7
	movwf	HI_TEMP
	call	add16
	return

RS_aziele			; GS-232 Format
        bcf     Rotor_Flag,6    ; AZ aktiv
        bcf     Rotor_Flag,7    ; EL aktiv
        LCDcmd  0x88
        LCDchar 'A'
        LCDchar 'Z'
        call    SERV_input1
        LCDchar ' '
        LCDchar ' '
        call    A_korr
        cpy16	LO,AZ_LO
        LCDcmd  0x80
        LCDchar 'E'
        LCDchar 'L'
        call    SERV_input1
	goto	RS_elevat1

RS_azkorr
        LCDcmd  LCDCLR
        LCDcmd  0x88            ; 1.Zeile/8. Position
        call    SERV_input
	cpy16	LO,AZkorr_LO
        goto    Loop

RS_elkorr
        LCDcmd  0x80            ; 1.Zeile/1. Position
        call    SERV_input
	cpy16	LO,ELkorr_LO
	clrw			;ab EEPROM Adresse 0
        call    put_EElat       ;speichere 4 Korrekturwerte im EEprom 
        goto    Loop

RS_hysterese
        LCDcmd  0x80            ; 1.Zeile/1. Position
	movlw	4
	movwf	EEADR
        call    SERV_input
	movfw	LO
	movwf	Rotor_hyst
        call    EE_Write
        goto    Loop

RS_message
        call    GET_next        ; 2ter Teil Prefix entfaellt
        LCDline 1
        movlw   .16
        movwf   stringlen
msg_loop
        call    GET_next
        movlw   ';'
        subwf   fifo_char,w
        bz      blank
        movfw   fifo_char
        LCDw
        decfsz  stringlen,f
        goto    msg_loop
        goto    Loop
blank
        LCDchar ' '
        decfsz  stringlen,f
        goto    blank
        goto    Loop



GET_next
        movfw   fifo_len
        btfsc   STATUS,Z
        goto    GET_next
        call    FIFOget		; Zeichen ist noch in w
        sublw   '.'             ; der Test strt ( hoffentlich) sonst keinen
        return

SERV_input  movfw   fifo_char	; 1ter Buchstabe ist schon da und bereits getestet
        LCDw			; aufs Display
        call    GET_next	; 2ter Buchstabe
        movfw   fifo_char	; wird nicht getestet
        LCDw
SERV_input1			; Label fr GS-232 Format
	clrf TEMP3		; wir max 3 Ziffern,den Punkt
	clrf	char_cnt	; und die Dezimale
;
        call    GET_next
        bz      SERV_fertig     ;'.' empfangen
	incf	char_cnt,f	;1 Ziffer empfangen	
        movfw   fifo_char
        movwf   TEMP0
;
        call    GET_next
        bz      SERV_fertig     ;'.' empfangen
	incf	char_cnt,f	;2 Ziffern empfangen	
        movfw   fifo_char
        movwf   TEMP1
;
        call    GET_next
        bz      SERV_fertig     ;'.' empfangen
	incf	char_cnt,f	;3 Ziffern empfangen	
        movfw   fifo_char
        movwf   TEMP2
        call    GET_next
        bnz      SERV_exit	;kein '.' empfangen, Ausgang
SERV_fertig
        call    GET_next
        movfw   fifo_char
        andlw   0x0f            ; wir arbeiten intern in 10tel Grad
        movwf   TEMP3           ; um A/D Genauigkeit auszunutzen
SERV_exit
        call    shufflechar
        call    ASC2BIN         ; konvertiere
        call    LCDval16	; und Display, nur die Stellen vor dem Punkt
        return

;************************************************************************

ADC_service
;Azimuth
;        LCDline 2
        LCDcmd  0xc8            ; 2.Zeile/8. Position
        LCDchar 'a'
        LCDchar 'z'
        bcf     ADC_Var,7	; A/D-Kanal 0 fr Azimuth
        call    LTC98;
        shl16   LO
        cpy16   AZkorr_LO,LO_TEMP
        call    ADC_korr	; Korrektur um Antennenoffset
        LCDcmd  0xce            ; 2.Zeile/15. Position
; Auswertung fuer Rotorsteuerung
	cpy16	AZ_LO,LO_TEMP
        call    sub16           ; Ist_Wert - Soll_Wert
        movlw   'R'             ; wir vermuten negativ: drehe rechts
        btfsc   HI,7            ; Ergebnis negativ ?
        goto    AZ_right        ; ja
        clrf    HI_TEMP         ; Teste auf Stop
        movfw   Rotor_hyst       ; Hysterese in zehntel Grad
        movwf   LO_TEMP
        call    sub16
        movlw   'L'             ; wir vermuten positiv: drehe links
        btfsc   HI,7            ; Ergebnis negativ ?
        movlw   'A'             ; im richtigen Bereich: stop Azimuth
AZ_right
        movwf   AZ_char
        LCDw
;
Elevation
        LCDcmd  0xc0            ; 2.Zeile/1. Position
        LCDchar 'e'
        LCDchar 'l'
	bsf	ADC_Var,7	; A/D-Kanal 1 fr Elevation
        call    LTC98
        shr16   LO
        cpy16   ELkorr_LO,LO_TEMP
        call    ADC_korr	; Korrektur um Antennenoffset
        LCDcmd  0xc6            ; 2.Zeile/7. Position
; Auswertung fuer Rotorsteuerung
	cpy16	EL_LO,LO_TEMP
        call    sub16           ; Ist_Wert - Soll_Wert
        movlw   'U'             ; wir vermuten negativ: drehe hoch
        btfsc   HI,7            ; Ergebnis negativ ?
        goto    EL_up           ; ja
        clrf    HI_TEMP         ; Teste auf Stop
        movfw   Rotor_hyst       ; Hysterese in zehntel Grad
        movwf   LO_TEMP
        call    sub16
        movlw   'D'             ; wir vermuten positiv: drehe runter
        btfsc   HI,7            ; Ergebnis negativ ?
        movlw   'E'             ; im richtigen Bereich: stop Elevation
EL_up
        movwf   EL_char
        LCDw
	return

ADC_korr			; wir wollen unbedingt die tatschliche Richtung
        cpy16     LO,ADLO_sav	; anzeigen, daher fr das Display Korrektur um
        call    add16		; Antennenoffset ( = Position bei Nullanschlag )
        movlw   .16
        movwf   LO_TEMP
        movlw   .14
        movwf   HI_TEMP
        call    cmpless16	; wir rechnen modulo 360
        btfss   HI_RESULT,7	; daher gegebenenfalls
        call    sub16		; Korrektur
        LCDval_16               ; Korrektur nur !! fuer Display
        cpy16     ADLO_sav,LO	; es wird mit ADC-Werten weitergerechnet
        return

;************************************************************************

Rotor_Move
        btfsc   Rotor_Flag,6            ; kein L/R, wenn nicht aktiv
        goto    Rotor_Move_EL
	bcf	Rotor_Flag,3
	bcf	Rotor_Flag,2
        movlw   'R'
        subwf   AZ_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,3            ; drehe rechts
        movlw   'L'
        subwf   AZ_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,2            ; drehe links
        movlw   'A'
        subwf   AZ_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,6            ; mache AZ inaktiv
Rotor_Move_EL
        btfsc   Rotor_Flag,7            ; kein U/D, wenn nicht aktiv
        goto Rotor_Move_c
	bcf	Rotor_Flag,1
	bcf	Rotor_Flag,0
        movlw   'D'
        subwf   EL_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,1            ; drehe nach unten
        movlw   'U'
        subwf   EL_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,0            ; drehe nach oben
        movlw   'E'
        subwf   EL_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,7            ; mache EL inaktiv
Rotor_Move_c
        movfw   Rotor_Flag
        movwf   PORTA			; neues Kommando an Rotor
        return


;************************************************************************
show_korrektur
	LCDline	1
        LCDchar 'e'
        LCDchar 'k'
	cpy16	ELkorr_LO,LO
        LCDval_16
        LCDchar 'a'
        LCDchar 'k'
	cpy16	AZkorr_LO,LO
        LCDval_16
        LCDchar 'h'
        movfw   Rotor_hyst
        LCDval_08
        goto    Loop

;************************************************************************
; set my default EEPROM:
; location  0: Azimut Offset der Antenne
; location  2: Elevation Offset der Antenne
; location  4: Hysterese der Antennenbewegung
;
        org 0x2100
EE_str0 DE      0x00,0x00,0x00,0x00,.20 ; Azimut L/H,Elevation L/H-offset,Hysterese
EE_str1 DE      "R","o","t","o","r"," ","I","n","t","e","r","f","a","c","e"," ",0
;************************************************************************

	End		; End of program


